<!---
|| APPLICATION ||
Application: General (Farcry)

|| LEGAL ||
Planning SA (http://www.planning.sa.gov.au)
Copyright:  Copyright Planning SA, Department of Transport and Urban Planning 2004.
All Rights Reserved.  All works and information displayed are subject to copyright.  For
the reproduction or publication beyond that permitted by the Copyright Act 1968 (Cwlth)
written permission must be sought from the Department.

Although every effort has been made to ensure the accuracy of the information displayed,
the Department, its agents, officers and employees make no representations, either express
or implied, that the information displayed is accurate or fit for any purpose and expressly
disclaims all liability for loss or damage arising from reliance upon the information displayed.

|| VERSION CONTROL ||
$RCSfile: checkLinkObjects.cfm,v $
$Header: I:\\backup\\cvs/atlassa/system/dmCron/checkLinkObjects.cfm,v 1.4 2004/10/26 01:44:46 whiterd Exp $
$Author: whiterd $
$Date: 2004/10/26 01:44:46 $
$Name:  $
$Revision: 1.4 $

|| DESCRIPTION ||
$Description: Emails a link validity report $
$TODO: check false errors with links involving redirection, /farcry/index.cfm?section=site&rootobjectid= seems to need the parent id rather than the dmlink object's id - is there a way to target the admin's site frameset with just the object id? $

|| DEVELOPER ||
$Developer: David Whiterod (whiterod.david@saugov.sa.gov.au) Planning SA$

Notes:
  * 1.1 Placed under CVS control, HTML Formatting based on statsOverviewReport.cfm (from core)
  * 1.2 Enhanced formatting
  * 1.3 Email links are checked for valid structure not for existance
  * 1.4 added parameter validation

|| ATTRIBUTES ||

emailTo        - REQUIRED emailaddress (comma-separated list) for report addressing
httpTimeout    - OPTIONAL timeout for check link check (in seconds; default: 10)
ProxyServer    - OPTIONAL used for the cfhttp call
ProxyPort      - OPTIONAL used for the cfhttp call (default: 80)
proxyUser      - OPTIONAL used for the cfhttp call
proxyPassword  - OPTIONAL used for the cfhttp call

If you supply ProxyServer argument you must also supply ProxyPort
If you supply proxyUser argument you must also supply proxyPassword


--->
<!--- @@displayname: Create link object report (dmLink objects) --->

<cfsetting enablecfoutputonly="Yes" requestTimeout="600">

<cfparam name="url.emailTo" default="">
<cfparam name="url.httpTimeout" default="10">
<cfparam name="url.ProxyServer" default="">
<cfparam name="url.ProxyPort" default="80">
<cfparam name="url.proxyUser" default="">
<cfparam name="url.proxyPassword" default="">

<cfimport taglib="/farcry/fourq/tags/" prefix="q4">
<cfparam name="stargs.typename" default="dmXMLExport">

<!--- Parameter validation --->
<cfset Variables.bArgError = false>

<cfif NOT Len(Trim(url.emailTo))>
  <!--- no email supplied --->
  <cfoutput>
    <span class="frameMenuBullet">&raquo;</span> Warning: No email supplied!
    <p>Supply argument as: emailTo=somedude@mydomain.com.au,anotherdude@anotherdomain.org.au, ...</p>
  </cfoutput>
  <cfset Variables.bArgError = true>
</cfif>

<!--- If ProxyServer is supplied then so to must be ProxyPort --->
<cfif Len(Trim(url.ProxyServer))>
  <cfif NOT Len(Trim(url.ProxyPort))>
    <!--- no ProxyPort supplied --->
    <cfoutput>
      <span class="frameMenuBullet">&raquo;</span> Warning: Argument error!
      <p>If supply the ProxyServer argument you must also supply the ProxyPort</p>
    </cfoutput>
    <cfset Variables.bArgError = true>
  </cfif>
</cfif>

<!--- If proxyUser is supplied then so to must be proxyPassword --->
<cfif Len(Trim(url.proxyUser))>
  <cfif NOT Len(Trim(url.proxyPassword))>
    <!--- no proxyPassword supplied --->
  <cfoutput>
    <span class="frameMenuBullet">&raquo;</span> Warning: Argument error!
    <p>If you supply proxyUser argument you must also supply the proxyPassword</p>
  </cfoutput>
    <cfset Variables.bArgError = true>
  </cfif>
</cfif>

<cfif Variables.bArgError>
  <cfdump var="#URL#" label="Parameters supplied">
  <cfoutput>
    <h3>Arguments</h3>
    <p>emailTo - REQUIRED emailaddress (comma-separated list) for report addressing</p>
    <p>httpTimeout - OPTIONAL timeout for check link check (in seconds; default: 10)</p>
    <p>ProxyServer - OPTIONAL used for the cfhttp call</p>
    <p>ProxyPort - OPTIONAL used for the cfhttp call (default: 80)</p>
    <p>proxyUser - OPTIONAL used for the cfhttp call</p>
    <p>proxyPassword - OPTIONAL used for the cfhttp call</p>
  </cfoutput>
  <cfabort>
</cfif>

<!--- Required functions with liberal helpings from CFLib.org (http://www.cflib.org) --->
<cfscript>
  /**
   * Tests passed value to see if it is a valid e-mail address (supports subdomain nesting and new top-level domains).
   * Update by David Kearns to support '
   * SBrown@xacting.com pointing out regex still wasn't accepting ' correctly.
   *
   * @param str    The string to check. (Required)
   * @return Returns a boolean.
   * @author Jeff Guillaume (jeff@kazoomis.com)
   * @version 2, August 15, 2002
   */
  function IsEmail(str) {
    //supports new top level tlds
    if (REFindNoCase("^['_a-z0-9-]+(\.['_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*\.(([a-z]{2,3})|(aero|coop|info|museum|name))$",str)) return TRUE;
      else return FALSE;
  }

  /**
   * Converts a structure of structures to a CF Query.
   *
   * @param theStruct    The structure to translate. (Required)
   * @param primaryKey    The query column name to use for the primary key. (Required)
   * @return Returns a query.
   * @author Shawn Seley (shawnse@aol.com)
   * @version 1, June 28, 2002
   */
  function StructOfStructuresToQuery(theStruct, primaryKey){
    var primary_key_list   = StructKeyList(theStruct);
    var field_list         = StructKeyList(theStruct[ListFirst(primary_key_list)]);
    var num_rows           = ListLen(primary_key_list);
    var the_query          = QueryNew(primaryKey & "," & field_list);
    var primary_key_value  = "";
    var field_name         = "";
    var the_value          = "";
    var row                = 0;
    var col                = 0;

    for(row=1; row LTE num_rows; row=row+1) {
      QueryAddRow(the_query);
      primary_key_value = ListGetAt(primary_key_list, row);
      QuerySetCell(the_query, primaryKey, primary_key_value);
      for(col=1; col LTE ListLen(field_list); col=col+1) {
        field_name = ListGetAt(field_list, col);
        the_value  = theStruct[primary_key_value][field_name];
        QuerySetCell(the_query, field_name, the_value);
      }
    }
    return(the_query);
  }
</cfscript>

<!--- Start HTML output --->
<cfoutput><span class="FormTitle">Checking links</span><p></p></cfoutput>

<cfscript>
  oLink = createObject("component",application.types['dmLink'].typepath);
  //get all link objects
  stLinkObjects = oLink.getMultiple(dsn=application.dsn,dbowner=application.dbowner);
</cfscript>
<!--- Perform checks --->
<cfloop collection="#stLinkObjects#" item="key">
  <!--- fill structure with default values --->
  <cfscript>
    stLinkObjects[key].checkStatus = "Not Checked";
    stLinkObjects[key].checkErrorDetail = "";
    stLinkObjects[key].checkStatusCode = "Excluded";
  </cfscript>

  <cfoutput>Checking <a href="#stLinkObjects[key].link#" title="View Object">#stLinkObjects[key].Title#</a> ...</cfoutput>
  <cfflush>

  <cfif FindNoCase("email", stLinkObjects[key].displayMethod)>
    <!--- check to valid e-mail address structure --->
    <cfscript>
      Variables.validEmail = IsEmail(stLinkObjects[key].link);
      stLinkObjects[key].linkType = "Email";
      stLinkObjects[key].checkStatusCode = Variables.validEmail;
      stLinkObjects[key].checkErrorDetail = "";
    </cfscript>
    <cfoutput>Valid Email: #Variables.validEmail#</cfoutput>
  <cfelse>
    <!--- Check website links --->
    <cfif Len(Trim(proxyUser))>
     <cfhttp
       url="#stLinkObjects[key].link#"
       method="head"
       timeout="#url.httpTimeout#"
       proxyserver="#URL.ProxyServer#"
       proxyport="#URL.ProxyPort#"
       proxyUser="#URL.proxyUser#"
       proxyPassword="#URL.proxyPassword#" />
    <cfelseif Len(Trim(ProxyServer))>
      <cfhttp
       url="#stLinkObjects[key].link#"
       method="head"
       timeout="#url.httpTimeout#"
       proxyserver="#URL.ProxyServer#"
       proxyport="#URL.ProxyPort#" />
    <cfelse>
      <cfhttp
       url="#stLinkObjects[key].link#"
       method="head"
       timeout="#url.httpTimeout#" />
    </cfif>

    <cfscript>
      stLinkObjects[key].linkType = "Website";
      stLinkObjects[key].checkStatusCode = cfhttp.StatusCode;
      stLinkObjects[key].checkErrorDetail = cfhttp.errorDetail;
    </cfscript>

    <cfoutput>#cfhttp.StatusCode# #cfhttp.errorDetail#</cfoutput>

  </cfif>

  <cfoutput><br /></cfoutput>
  <cfflush>
  <cfset stLinkObjects[key].checkStatus = "Checked">

</cfloop>

<!--- Convert Structure to query --->
<!--- StructOfStructuresToQuery requires a primary key send another name (i.e. xxxOID) so not to have two objectid items in the returned query --->
<cfset Variables.qLinkCheckResults = StructOfStructuresToQuery(stLinkObjects,"xxxOID")>

<!--- Rip apart new query (for output formatting) --->
<cfquery name="qRecordsByType" dbType="query">
  select count(*) as GroupTotal
  from Variables.qLinkCheckResults
  Group By linkType
  Order by linkType
</cfquery>

<cfquery name="qWebsiteLinks" dbType="query">
  select *
  from Variables.qLinkCheckResults
  where linkType = 'Website'
  Order by checkStatusCode DESC, Title
</cfquery>

<cfquery name="qEmailLinks" dbType="query">
  select *
  from Variables.qLinkCheckResults
  where linkType = 'Email'
  Order by checkStatusCode, Title
</cfquery>

<cfmail to="#url.emailTo#" from="#application.config.general.adminEmail#" subject="#application.config.general.siteTitle# Link Report" type="HTML">
  <style>
    .FormTitle {
      font: bold 12px Verdana, Geneva, Arial, Helvetica, sans-serif;
      color:##666;
      text-transform: uppercase;
      letter-spacing: .1em;
    }

    .FormSubTitle {
      font: bold 11px Verdana, Geneva, Arial, Helvetica, sans-serif;
      color:##666;
      letter-spacing: .1em;
    }

    .dataHeader {
      background-color : ##666;
      color: ##ededed;
    }

    .dataOddRow {
      background-color : ##ccc;
      color: ##333;
    }

    .dataEvenRow {
      background-color : ##ededed;
      color: ##333;
    }

    .dataWarningRow {
      background-color: ##FFCC66;
      color: ##333;
    }
    .dataErrorRow {
      background-color: ##FF6600;
      color: ##333;
    }

    table, td, div {
      font: 10px Verdana, Geneva, Arial, Helvetica, sans-serif;
    }
  </style>

  <h1 class="formtitle">Link Report - Generated #dateformat(now(),"dd-mmm-yyyy hh:mm")#</h1>
  <table border="0" cellpadding="0" cellspacing="0">
    <tr>
      <td valign="top">
        <p class="FormSubTitle">Link Summary</p>
        <table cellpadding="5" cellspacing="0" border="1" style="margin-left:30px;">
          <cfif qWebsiteLinks.RecordCount gt 0>
            <tr>
              <td class="dataOddRow"><a href="##website">Website links</a></td>
              <td class="dataOddRow">#qWebsiteLinks.RecordCount#</td>
            </tr>
          </cfif>
          <cfif qEmailLinks.RecordCount gt 0>
            <tr>
              <td class="dataEvenRow"><a href="##email">Email links</a></td>
              <td class="dataEvenRow">#qEmailLinks.RecordCount#</td>
            </tr>
          </cfif>
          <tr>
            <th class="dataOddRow">Total links</th>
            <th class="dataOddRow">#StructCount(stLinkObjects)#</th>
          </tr>
        </table>
      </td>
      <td width="10px">&nbsp;</td>
      <td valign="top">
        <p class="FormSubTitle">Report Legend</p>
        <table cellpadding="5" cellspacing="0" border="1" width="100" style="margin-left:30px;">
          <tr>
            <th class="dataWarningRow">Warning</th>
          </tr>
          <tr>
            <th class="dataErrorRow">Error</th>
          </tr>
        </table>
      </td>
    </tr>
  </table>

  <cfif qWebsiteLinks.RecordCount gt 0>
    <a name="website"></a>
    <p class="FormSubTitle">HTML Link Report</p>
    <table cellpadding="5" cellspacing="0" border="1" style="margin-left:30px;">
      <tr>
        <th class="dataheader" align="left">ObjectID</th>
        <th class="dataheader">Title</th>
        <th class="dataheader">Status Code</th>
        <th class="dataheader">Error Details</th>
        <th class="dataheader">Last Updated By</th>
        <th class="dataheader">Last Updated</th>
        <th class="dataheader">Address</th>
      </tr>

      <cfloop query="qWebsiteLinks">
        <cfif qWebsiteLinks.CurrentRow MOD 2>
          <cfset Variables.rowClass = "dataOddRow">
        <cfelse>
          <cfset Variables.rowClass = "dataEvenRow">
        </cfif>

        <!--- set warning and error markup --->
        <cfswitch expression="#Left(qWebsiteLinks.CHECKSTATUSCODE,1)#">

          <cfcase value="1,2,3">
            <cfset tmp = "">
          </cfcase>

          <cfcase value="4">
            <cfset Variables.rowClass = "dataWarningRow">
          </cfcase>

          <cfcase value="5">
            <cfset Variables.rowClass = "dataErrorRow">
          </cfcase>

          <cfdefaultcase>
            <cfset Variables.rowClass = "dataErrorRow">
          </cfdefaultcase>

        </cfswitch>

        <tr class="#Variables.rowClass#">
          <td align="left"><a href="#application.config.general.ADMINSERVER##application.url.farcry#/edittabOverview.cfm?objectid=#qWebsiteLinks.ObjectID#" title="Edit Object">#qWebsiteLinks.ObjectID#</a></td>
          <td>#qWebsiteLinks.Title#</td>
          <td>#qWebsiteLinks.CHECKSTATUSCODE#&nbsp;</td>
          <td>#qWebsiteLinks.CHECKERRORDETAIL#&nbsp;</td>
          <td>#qWebsiteLinks.lastupdatedby#</td>
          <td>#qWebsiteLinks.datetimelastupdated#</td>
          <td><a href="#qWebsiteLinks.link#" title="View Site">#qWebsiteLinks.link#</a></td>
        </tr>
      </cfloop>
    </table>
  </cfif>

  <cfif qEmailLinks.RecordCount gt 0>
    <a name="email"></a>
    <p class="FormSubTitle">Email Link Report</p>
    <table cellpadding="5" cellspacing="0" border="1" width="500" style="margin-left:30px;">
      <tr>
        <th class="dataheader" align="left">ObjectID</th>
        <th class="dataheader">Title</th>
        <th class="dataheader">Valid address</th>
        <th class="dataheader">Last updated by</th>
        <th class="dataheader">Last updated</th>
        <th class="dataheader">Address</th>
      </tr>
      <cfloop query="qEmailLinks">
        <cfif qEmailLinks.CurrentRow MOD 2>
          <cfset Variables.rowClass = "dataOddRow">
        <cfelse>
          <cfset Variables.rowClass = "dataEvenRow">
        </cfif>

        <!--- set warning and error --->
        <cfswitch expression="#qEmailLinks.CHECKSTATUSCODE#">

          <cfcase value="TRUE">
            <cfset tmp = "">
          </cfcase>

          <cfcase value="FALSE">
            <cfset Variables.rowClass = "dataErrorRow">
          </cfcase>

          <cfdefaultcase>
            <cfset Variables.rowClass = "dataWarningRow">
          </cfdefaultcase>

        </cfswitch>

        <tr class="#Variables.rowClass#">
          <td align="left"><a href="#application.config.general.ADMINSERVER##application.url.farcry#/edittabOverview.cfm?objectid=#qEmailLinks.ObjectID#" title="Edit Object">#qEmailLinks.ObjectID#</a></td>
          <td>#qEmailLinks.Title#</td>
          <td>#qEmailLinks.CHECKSTATUSCODE#&nbsp;</td>
          <td>#qEmailLinks.lastupdatedby#</td>
          <td>#qEmailLinks.datetimelastupdated#</td>
          <td><a href="mailto:#qEmailLinks.link#" title="Send email">#qEmailLinks.link#</a></td>
        </tr>
      </cfloop>
    </table>
  </cfif>

</cfmail>

<cfoutput><p>Email sent to: #url.emailTo#</p></cfoutput>
<cfoutput><p>All done.</p></cfoutput>
<!--- <cfdump var="#Variables.qLinkCheckResults#"> --->
<!--- <cfdump var="#stLinkObjects#"> --->
<cfsetting enablecfoutputonly="No">
